"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const ontology_ts_crypto_1 = require("ontology-ts-crypto");
const path = require("path");
const consts_1 = require("../consts");
const punicaException_1 = require("../exception/punicaException");
// tslint:disable:no-console
function loadConfigFile(configFilePath) {
    try {
        const f = fs.readFileSync(configFilePath);
        return JSON.parse(f.toString());
    }
    catch (e) {
        throw punicaException_1.configFileNotFound();
    }
}
exports.loadConfigFile = loadConfigFile;
function loadNetwork(configDirPath, networkKey, debug) {
    const configFilePath = path.join(configDirPath, 'punica-config.json');
    const config = loadConfigFile(configFilePath);
    try {
        if (networkKey === undefined) {
            if (config.defaultNet !== undefined) {
                networkKey = config.defaultNet;
            }
            else {
                throw punicaException_1.otherError('No network specified');
            }
        }
    }
    catch (e) {
        throw punicaException_1.configFileError();
    }
    try {
        const network = config.networks[networkKey];
        const address = `${network.host}:${network.port}`;
        if (network === undefined) {
            throw punicaException_1.otherError(`There is no network called ${networkKey} in config file.`);
        }
        console.log(`Using network '${networkKey}'.`);
        return address;
    }
    catch (e) {
        throw punicaException_1.configFileError();
    }
}
exports.loadNetwork = loadNetwork;
function loadConfig(projectDir, configDir) {
    let configPath;
    if (configDir !== undefined) {
        configPath = path.join(projectDir, configDir);
        if (fs.existsSync(configPath)) {
            if (!fs.statSync(configPath).isFile()) {
                throw punicaException_1.otherError(`${configPath} is not a file`);
            }
        }
        else {
            configPath = path.join(projectDir, 'contracts', configDir);
            if (!fs.existsSync(configPath)) {
                throw punicaException_1.otherError(`${configPath} does not exist`);
            }
        }
    }
    else {
        configPath = path.join(projectDir, 'contracts', consts_1.DEFAULT_CONFIG);
    }
    if (!fs.existsSync(configPath)) {
        throw punicaException_1.otherError(`${configPath} does not exist`);
    }
    return loadConfigFile(configPath);
}
exports.loadConfig = loadConfig;
function loadDeploy(projectDir, configDir) {
    const config = loadConfig(projectDir, configDir);
    let deployInformation = config.deployInformation;
    if (deployInformation === undefined || typeof deployInformation !== 'object') {
        deployInformation = config.deployConfig;
        if (deployInformation === undefined || typeof deployInformation !== 'object') {
            throw punicaException_1.configFileError();
        }
    }
    return deployInformation;
}
exports.loadDeploy = loadDeploy;
function loadPassword(projectDir, configDir, address) {
    const config = loadConfig(projectDir, configDir);
    const passwords = config.password;
    if (passwords !== undefined && typeof passwords === 'object') {
        return passwords[address];
    }
    return undefined;
}
exports.loadPassword = loadPassword;
function loadInvoke(projectDir, configDir) {
    const config = loadConfig(projectDir, configDir);
    const invokeConfig = config.invokeConfig;
    if (invokeConfig === undefined || typeof invokeConfig !== 'object') {
        throw punicaException_1.configFileError();
    }
    return invokeConfig;
}
exports.loadInvoke = loadInvoke;
function loadWallet(projectDir, walletFileName) {
    let walletPath;
    if (walletFileName === undefined) {
        const walletDirPath = path.join(projectDir, 'wallet');
        const dir = fs.readdirSync(walletDirPath);
        if (dir.length === 1) {
            walletPath = path.join(walletDirPath, dir[0]);
        }
        else {
            throw punicaException_1.walletFileUnspecified();
        }
    }
    else {
        walletPath = path.join(projectDir, walletFileName);
        if (!fs.existsSync(walletPath)) {
            walletPath = path.join(projectDir, 'wallet', walletFileName);
            if (!fs.existsSync(walletPath)) {
                throw punicaException_1.otherError(`${walletPath} does not exist`);
            }
        }
    }
    try {
        const f = fs.readFileSync(walletPath, 'utf8');
        return [ontology_ts_crypto_1.Wallet.deserializeJson(f), walletPath];
    }
    catch (e) {
        throw punicaException_1.walletFileError();
    }
}
exports.loadWallet = loadWallet;
function loadAccount(wallet, address) {
    if (address === undefined) {
        address = wallet.defaultAccountAddress;
    }
    for (const account of wallet.accounts) {
        if (account.address.toBase58() === address) {
            return account;
        }
    }
    throw punicaException_1.otherError('Payer account not found');
}
exports.loadAccount = loadAccount;
//# sourceMappingURL=configLoader.js.map